#include <iostream>

#include "cBoard.h"
#include "constants.h"
#include "functions.h"

// f(n) = [number of 3-lengths open for pc] - [number of 3-lengths open for player]
int cBoard::evaluate() {
    int rowEvaluations[8];
    int counter = 0;
    int pcCount = 0, playerCount = 0;
    bool pcWin = false, playerWin = false;

    for (int mode = 0; mode < 2; mode++) {
        for (int pos = 0; pos < 3; pos++) {
            switch (mode) {
                case 0: // Horizontal
                    rowEvaluations[counter] = analyzeTriplet(field[0][pos], field[1][pos], field[2][pos]);
                    break;
                case 1: // Vertical
                    rowEvaluations[counter] = analyzeTriplet(field[pos][0], field[pos][1], field[pos][2]);
                    break;
            }
            counter++;
        }
    }

    rowEvaluations[counter] = analyzeTriplet(field[0][0], field[1][1], field[2][2]);
    rowEvaluations[counter+1] = analyzeTriplet(field[0][2], field[1][1], field[2][0]);

    for (int i = 0; i < 8; i++) {
        switch (rowEvaluations[i]) {
            case INT_PC:
                pcCount++;
                break;
            case INT_PLAYER:
                playerCount++;
                break;
            case INT_BOTH:
                playerCount++;
                pcCount++;
                break;
            case INT_PCWIN:
                pcWin = true;
                break;
            case INT_PLAYERWIN:
                playerWin = true;
                break;
        }
    }

    if (pcWin) {
        score = INT_PCWIN;
    } else if (playerWin) {
        score = INT_PLAYERWIN;
    } else {
        score = pcCount - playerCount;
    }

    return score;
}

int cBoard::setField(int tl, int tm, int tr, int ml, int mm, int mr, int bl, int bm, int br) {
    field[0][0] = tl;
    field[1][0] = tm;
    field[2][0] = tr;
    field[0][1] = ml;
    field[1][1] = mm;
    field[2][1] = mr;
    field[0][2] = bl;
    field[1][2] = bm;
    field[2][2] = br;

    evaluate();

    return 0;
}

cBoard::cBoard(int tl, int tm, int tr, int ml, int mm, int mr, int bl, int bm, int br) {
    field[0][0] = tl;
    field[1][0] = tm;
    field[2][0] = tr;
    field[0][1] = ml;
    field[1][1] = mm;
    field[2][1] = mr;
    field[0][2] = bl;
    field[1][2] = bm;
    field[2][2] = br;

    evaluate();
}

int cBoard::setField(cBoard* inputBoard) {
    for (int x = 0; x < 3; x++) {
        for (int y = 0; y < 3; y++) {
            field[x][y] = inputBoard->getPos(x, y);
        }
    }

    evaluate();

    return 0;
}

cBoard::cBoard(cBoard* inputBoard) {
    for (int x = 0; x < 3; x++) {
        for (int y = 0; y < 3; y++) {
            field[x][y] = inputBoard->getPos(x, y);
        }
    }

    evaluate();
}

cBoard::cBoard() {
    for (int y = 0; y < 3; y++) {
        for (int x = 0; x < 3; x++) {
            field[x][y] = INT_EMPTY;
        }
    }

    evaluate();
}

int cBoard::setPos(int x, int y, int what) {
    field[x][y] = what;

    evaluate();

    return 0;
}

int cBoard::setPos(int x, int y, bool what) {
    if (what == TURN_PC) {
        field[x][y] = INT_PC;
    } else if (what == TURN_PLAYER) {
        field[x][y] = INT_PLAYER;
    }

    evaluate();

    return 0;
}

int cBoard::getPos(int x, int y) {
    int output = field[x][y];

    return output;
}

int cBoard::getScore() {
    return score;
}

int cBoard::print() {
    int tSpot;
    int counter = 0;

    std::cout << "+---+---+---+\n";
    for (int y = 0; y < 3; y++) {
        std::cout << "|   |   |   |\n";
        std::cout << "| ";
        for (int x = 0; x < 3; x++) {
            counter++;
            tSpot = field[x][y];

            switch (tSpot) {
            case INT_EMPTY:
                std::cout << counter;
                break;
            case INT_PC:
                std::cout << STRING_PC;
                break;
            case INT_PLAYER:
                std::cout << STRING_PLAYER;
                break;
            }
            std::cout << " | ";
        }
        std::cout << "\n|   |   |   |\n";
        std::cout << "+---+---+---+\n";
    }
    return 0;
}

bool cBoard::turnPossible() {
    bool possible = false;
    for (int y = 0; y < 3; y++) {
        for (int x = 0; x < 3; x++) {
            if (field[x][y] == 0) {
                possible = true;
            }
        }
    }

    return possible;
}
